@@ -29,6 +29,7 @@
 
 #include "gs-manager.h"
 #include "gs-window.h"
+#include "gs-theme-manager.h"
 #include "gs-job.h"
 #include "gs-grab.h"
 #include "gs-fade.h"
@@ -45,6 +46,8 @@ struct GSManagerPrivate
         GSList      *windows;
         GHashTable  *jobs;
 
+        GSThemeManager *theme_manager;
+
         /* Policy */
         glong        lock_timeout;
         glong        cycle_timeout;
@@ -52,10 +55,12 @@ struct GSManagerPrivate
 
         guint        lock_enabled : 1;
         guint        logout_enabled : 1;
+        guint        keyboard_enabled : 1;
         guint        user_switch_enabled : 1;
         guint        throttled : 1;
 
         char        *logout_command;
+        char        *keyboard_command;
 
         /* State */
         guint        active : 1;
@@ -64,11 +69,11 @@ struct GSManagerPrivate
         guint        fading : 1;
         guint        dialog_up : 1;
 
+        time_t       activate_time;
+
         guint        lock_timeout_id;
         guint        cycle_timeout_id;
 
-        time_t       activate_time;
-
         GSList      *themes;
         GSSaverMode  saver_mode;
         GSGrab      *grab;
@@ -89,10 +94,12 @@ enum {
         PROP_LOCK_ENABLED,
         PROP_LOGOUT_ENABLED,
         PROP_USER_SWITCH_ENABLED,
+        PROP_KEYBOARD_ENABLED,
         PROP_LOCK_TIMEOUT,
         PROP_CYCLE_TIMEOUT,
         PROP_LOGOUT_TIMEOUT,
         PROP_LOGOUT_COMMAND,
+        PROP_KEYBOARD_COMMAND,
         PROP_ACTIVE,
         PROP_THROTTLED,
 };
@@ -140,7 +147,6 @@ select_theme (GSManager *manager)
         return theme;
 }
 
-
 static GSJob *
 lookup_job_for_window (GSManager *manager,
                        GSWindow  *window)
@@ -206,16 +212,42 @@ manager_maybe_start_job_for_window (GSManager *manager,
 }
 
 static void
-cycle_job (GSWindow  *window,
-           GSJob     *job,
-           GSManager *manager)
+manager_select_theme_for_job (GSManager *manager,
+                              GSJob     *job)
 {
         const char *theme;
 
         theme = select_theme (manager);
 
+        if (theme != NULL) {
+                GSThemeInfo    *info;
+                const char     *command;
+
+                command = NULL;
+
+                info = gs_theme_manager_lookup_theme_info (manager->priv->theme_manager, theme);
+                if (info != NULL) {
+                        command = gs_theme_info_get_exec (info);
+                }
+
+                gs_job_set_command (job, command);
+
+
+                if (info != NULL) {
+                        gs_theme_info_unref (info);
+                }
+        } else {
+                gs_job_set_command (job, NULL);
+        }
+}
+
+static void
+cycle_job (GSWindow  *window,
+           GSJob     *job,
+           GSManager *manager)
+{
         gs_job_stop (job);
-        gs_job_set_theme (job, theme, NULL);
+        manager_select_theme_for_job (manager, job);
         manager_maybe_start_job_for_window (manager, window);
 }
 
@@ -380,6 +412,19 @@ gs_manager_set_lock_active (GSManager *manager,
         }
 }
 
+void
+gs_manager_get_lock_enabled (GSManager *manager,
+                             gboolean  *lock_enabled)
+{
+        if (lock_enabled != NULL) {
+                *lock_enabled = FALSE;
+        }
+
+        g_return_if_fail (GS_IS_MANAGER (manager));
+
+        *lock_enabled = manager->priv->lock_enabled;
+}
+
 void
 gs_manager_set_lock_enabled (GSManager *manager,
                              gboolean   lock_enabled)
@@ -407,6 +452,22 @@ gs_manager_set_logout_enabled (GSManager *manager,
         }
 }
 
+void
+gs_manager_set_keyboard_enabled (GSManager *manager,
+                                 gboolean   enabled)
+{
+        g_return_if_fail (GS_IS_MANAGER (manager));
+
+        if (manager->priv->keyboard_enabled != enabled) {
+                GSList *l;
+
+                manager->priv->keyboard_enabled = enabled;
+                for (l = manager->priv->windows; l; l = l->next) {
+                        gs_window_set_keyboard_enabled (l->data, enabled);
+                }
+        }
+}
+
 void
 gs_manager_set_user_switch_enabled (GSManager *manager,
                                     gboolean   user_switch_enabled)
@@ -500,6 +561,8 @@ void
 gs_manager_set_logout_command (GSManager  *manager,
                                const char *command)
 {
+        GSList *l;
+
         g_return_if_fail (GS_IS_MANAGER (manager));
 
         g_free (manager->priv->logout_command);
@@ -509,6 +572,31 @@ gs_manager_set_logout_command (GSManager  *manager,
         } else {
                 manager->priv->logout_command = NULL;
         }
+
+        for (l = manager->priv->windows; l; l = l->next) {
+                gs_window_set_logout_command (l->data, manager->priv->logout_command);
+        }
+}
+
+void
+gs_manager_set_keyboard_command (GSManager  *manager,
+                                 const char *command)
+{
+        GSList *l;
+
+        g_return_if_fail (GS_IS_MANAGER (manager));
+
+        g_free (manager->priv->keyboard_command);
+
+        if (command) {
+                manager->priv->keyboard_command = g_strdup (command);
+        } else {
+                manager->priv->keyboard_command = NULL;
+        }
+
+        for (l = manager->priv->windows; l; l = l->next) {
+                gs_window_set_logout_command (l->data, manager->priv->keyboard_command);
+        }
 }
 
 gboolean
@@ -618,6 +706,9 @@ gs_manager_set_property (GObject            *object,
         case PROP_LOGOUT_ENABLED:
                 gs_manager_set_logout_enabled (self, g_value_get_boolean (value));
                 break;
+        case PROP_KEYBOARD_ENABLED:
+                gs_manager_set_keyboard_enabled (self, g_value_get_boolean (value));
+                break;
         case PROP_USER_SWITCH_ENABLED:
                 gs_manager_set_user_switch_enabled (self, g_value_get_boolean (value));
                 break;
@@ -627,6 +718,9 @@ gs_manager_set_property (GObject            *object,
         case PROP_LOGOUT_COMMAND:
                 gs_manager_set_logout_command (self, g_value_get_string (value));
                 break;
+        case PROP_KEYBOARD_COMMAND:
+                gs_manager_set_keyboard_command (self, g_value_get_string (value));
+                break;
         case PROP_CYCLE_TIMEOUT:
                 gs_manager_set_cycle_timeout (self, g_value_get_long (value));
                 break;
@@ -659,6 +753,9 @@ gs_manager_get_property (GObject            *object,
         case PROP_LOGOUT_ENABLED:
                 g_value_set_boolean (value, self->priv->logout_enabled);
                 break;
+        case PROP_KEYBOARD_ENABLED:
+                g_value_set_boolean (value, self->priv->keyboard_enabled);
+                break;
         case PROP_USER_SWITCH_ENABLED:
                 g_value_set_boolean (value, self->priv->user_switch_enabled);
                 break;
@@ -668,6 +765,9 @@ gs_manager_get_property (GObject            *object,
         case PROP_LOGOUT_COMMAND:
                 g_value_set_string (value, self->priv->logout_command);
                 break;
+        case PROP_KEYBOARD_COMMAND:
+                g_value_set_string (value, self->priv->keyboard_command);
+                break;
         case PROP_CYCLE_TIMEOUT:
                 g_value_set_long (value, self->priv->cycle_timeout);
                 break;
@@ -810,6 +910,7 @@ gs_manager_init (GSManager *manager)
 
         manager->priv->fade = gs_fade_new ();
         manager->priv->grab = gs_grab_new ();
+        manager->priv->theme_manager = gs_theme_manager_new ();
 }
 
 static void
@@ -843,6 +944,7 @@ gs_manager_finalize (GObject *object)
 
         free_themes (manager);
         g_free (manager->priv->logout_command);
+        g_free (manager->priv->keyboard_command);
 
         remove_unfade_idle (manager);
         remove_timers (manager);
@@ -863,6 +965,7 @@ gs_manager_finalize (GObject *object)
 
         g_object_unref (manager->priv->fade);
         g_object_unref (manager->priv->grab);
+        g_object_unref (manager->priv->theme_manager);
 
         G_OBJECT_CLASS (gs_manager_parent_class)->finalize (object);
 }
@@ -937,6 +1040,8 @@ window_dialog_down_cb (GSWindow  *window,
         g_return_if_fail (manager != NULL);
         g_return_if_fail (GS_IS_MANAGER (manager));
 
+        gs_debug ("Handling dialog down");
+
         /* Regrab the mouse */
         gs_grab_move_to_window (manager->priv->grab,
                                 gs_window_get_gdk_window (window),
@@ -1085,14 +1190,11 @@ static void
 manager_show_window (GSManager *manager,
                      GSWindow  *window)
 {
-        GSJob      *job;
-        const char *theme;
+        GSJob *job;
 
         job = gs_job_new_for_widget (GTK_WIDGET (window));
 
-        theme = select_theme (manager);
-        gs_job_set_theme (job, theme, NULL);
-
+        manager_select_theme_for_job (manager, job);
         manager_add_job_for_window (manager, window, job);
 
         manager->priv->activate_time = time (NULL);
@@ -1242,6 +1344,8 @@ gs_manager_create_window (GSManager *manager,
                 gs_window_set_logout_enabled (window, manager->priv->logout_enabled);
                 gs_window_set_logout_timeout (window, manager->priv->logout_timeout);
                 gs_window_set_logout_command (window, manager->priv->logout_command);
+                gs_window_set_keyboard_enabled (window, manager->priv->keyboard_enabled);
+                gs_window_set_keyboard_command (window, manager->priv->keyboard_command);
 
                 connect_window_signals (manager, window);
 
@@ -1392,9 +1496,12 @@ gs_manager_deactivate (GSManager *manager)
         g_slist_free (manager->priv->windows);
         manager->priv->windows = NULL;
 
+        /* reset state */
         manager->priv->active = FALSE;
         manager->priv->activate_time = 0;
         manager->priv->lock_active = FALSE;
+        manager->priv->dialog_up = FALSE;
+        manager->priv->fading = FALSE;
 
         return TRUE;
 }
